//VFD Clock
//1.0 - initial release
//Kevin Rye - 2017
//Compiled on Arduino 1.6.10

#include <DS3231_Simple.h>
#include <Wire.h>
#include <DS3231.h>

DS3231 clock;
RTCDateTime dt;
RTCAlarmTime al1;

//time variables
int hh24;
int hh;
int mm;
int ss; // not used

//I/O
const int hoursButton = A0;
const int minsButton = A1;
const int alarmMenu = A2;
const int alarmEnable = A3;

//alarm
bool setAlarmActive = false; //set up default condition
bool inAlarmMenu = false; //set up default condition
int alarmHour = 0; //set up default condition
int adjAlarmHour;
int alarmMinute = 0; //set up default condition

//segments
const int segA = 0;
const int segB = 1;
const int segC = 2;
const int segD = 3;
const int segE = 4;
const int segF = 5;
const int segG = 6;
const int pm = 7;
const int alarmIndicator = 9;

//digits / grids
const int colon = 8;
const int min1s = 13; //grid 4
const int min10s = 12; //grid 3
const int hrs1s = 11; //grid 2
const int hrs10s = 10; //grid 1

//digit variables
byte grid4; //1s of mins
byte grid3; //10s of mins
byte grid2; //1s of hrs
byte grid1; //10s of hrs

//segment array (configured for upside-down displays)
byte segArray[11][7] = {{1, 1, 1, 1, 1, 1, 0}, // 0
  {0, 0, 0, 0, 1, 1, 0}, // 1
  {1, 1, 0, 1, 1, 0, 1}, // 2
  {1, 0, 0, 1, 1, 1, 1}, // 3
  {0, 0, 1, 0, 1, 1, 1}, // 4
  {1, 0, 1, 1, 0, 1, 1}, // 5
  {1, 1, 1, 1, 0, 1, 1}, // 6
  {0, 0, 0, 1, 1, 1, 0}, // 7
  {1, 1, 1, 1, 1, 1, 1}, // 8
  {1, 0, 1, 1, 1, 1, 1}, // 9
  {0, 0, 0, 0, 0, 0, 0} // off
};

volatile byte multiplexer = 0;

void setup() {

  clock.begin();
  clock.enableOutput(false); //disable square wave

  //set pin modes
  pinMode(segA, OUTPUT);
  pinMode(segB, OUTPUT);
  pinMode(segC, OUTPUT);
  pinMode(segD, OUTPUT);
  pinMode(segE, OUTPUT);
  pinMode(segF, OUTPUT);
  pinMode(segG, OUTPUT);
  pinMode(pm, OUTPUT);
  pinMode(alarmIndicator, OUTPUT);

  pinMode(colon, OUTPUT);
  pinMode(min1s, OUTPUT);
  pinMode(min10s, OUTPUT);
  pinMode(hrs1s, OUTPUT);
  pinMode(hrs10s, OUTPUT);

  pinMode(hoursButton, INPUT);
  pinMode(minsButton, INPUT);

  //set states
  digitalWrite(segA, LOW);
  digitalWrite(segB, LOW);
  digitalWrite(segC, LOW);
  digitalWrite(segD, LOW);
  digitalWrite(segE, LOW);
  digitalWrite(segF, LOW);
  digitalWrite(segG, LOW);
  digitalWrite(pm, LOW);
  digitalWrite(alarmIndicator, LOW);

}

void loop() {

  //get time
  dt = clock.getDateTime();
  hh24 = dt.hour; //used for PM check
  mm = dt.minute;
  ss = dt.second;

  //get alarm time
  al1 = clock.getAlarm1();
  alarmHour = al1.hour;
  alarmMinute = al1.minute;

  //pass RTC values into a new variable -
  //this way one variable stores time in 24 hour mode and another that can be converted to 12 hour mode
  //time is monitored 'in the background' in 24 hour mode using the RTC, but is displayed in an adjusted 12-hr format
  hh = hh24;

  //convert to 12 hr mode for display
  if (hh24 < 1) {
    hh = 12;
  }

  if ((hh24 > 12) && (hh24 < 24)) {
    hh = hh24 - 12;
  }

  //calculate the number to be displayed on each digit by disecting the time from the RTC
  //clock mode
  if (false == inAlarmMenu) {

    //hours
    grid1 = hh / 10;
    grid2 = hh % 10;

    //minutes
    grid3 = mm / 10;
    grid4 = mm % 10;

  } else { //in alarmmode

    adjAlarmHour = alarmHour;

    //convert to 12 hr mode for display
    if (alarmHour < 1) {
      adjAlarmHour = 12;
    }

    if ((alarmHour > 12) && (alarmHour < 24)) {
      adjAlarmHour = alarmHour - 12;
    }

    //alarm hours
    grid1 = adjAlarmHour / 10;
    grid2 = adjAlarmHour % 10;

    //alarm minutes
    grid3 = alarmMinute / 10;
    grid4 = alarmMinute % 10;
  }

  //turn digits off
  digitalWrite(min1s, LOW); //3rd digit - 1s of minutes
  digitalWrite(min10s, LOW); //4th digit - 10s of minutes
  digitalWrite(hrs1s, LOW); //5th digit - 1s of hours
  digitalWrite(hrs10s, LOW); //6th digit - 10s of hours
  digitalWrite(colon, LOW);
  digitalWrite(alarmIndicator, LOW);

  //do the multiplexing...
  multiplexer++;
  if (multiplexer == 6) multiplexer = 1; //digits 1, 2, 3, 4, then repeat

  // 1st digit
  if (multiplexer == 1) {

    displayDigit(grid1); //get value
    digitalWrite(hrs10s, HIGH); //turn on
    
    //turn off this digit if hour < 10 (leading zero)
    /*  if (hh < 10 ){
      digitalWrite(hrs10s, LOW); //turn off
      } else {
        digitalWrite(hrs10s, HIGH); //turn on
      }
    */
    
    //turn on PM light as appropriate in clock mode and alarm mode
    if (false == inAlarmMenu) {
      //clock mode
      if (hh24 > 11) {
        digitalWrite(pm, HIGH);
      }
      else {
        digitalWrite(pm, LOW);
      }

    } else {// in alarm mode
      if (alarmHour > 11) {
        digitalWrite(pm, HIGH);
      }
      else {
        digitalWrite(pm, LOW);
      }
    }

    digitalWrite(colon, LOW);

    //if alarm is set, turn on the indicator
    if (true == setAlarmActive) {
      digitalWrite(alarmIndicator, HIGH);
    } else {
      digitalWrite(alarmIndicator, LOW);
    }
  }

  // 2nd digit
  if (multiplexer == 2) {

    displayDigit(grid2); //get value
    digitalWrite(hrs1s, HIGH); //enable
    digitalWrite(pm, LOW);
    digitalWrite(colon, LOW);
  }

  //colon
  if (multiplexer == 3) {

    digitalWrite(colon, HIGH);
    digitalWrite(pm, LOW);

    //turn on center indicator if in alarm mnu
    if (true == inAlarmMenu) {
      digitalWrite(alarmIndicator, HIGH); //alarm indicator in this case is the one that is connected to the colon grid
    } else {
      digitalWrite(alarmIndicator, LOW);
    }
  }

  // 3rd digit
  if (multiplexer == 4) {

    displayDigit(grid3); //get value
    digitalWrite(min10s, HIGH); //enable
    digitalWrite(pm, LOW);
    digitalWrite(colon, LOW);
  }

  // 4th digit
  if (multiplexer == 5) {

    displayDigit(grid4); //get value
    digitalWrite(min1s, HIGH); //enable
    digitalWrite(pm, LOW);
    digitalWrite(colon, LOW);
  }

  //handle button presses for setting the time
  //hours button pressed
  if (digitalRead(hoursButton) == LOW) {

    //debounce
    delay(200);
    if (true == inAlarmMenu) {
      //set alarm hours
      alarmHour++;

      if (alarmHour > 23) {
        alarmHour = 0;
      }
      setAlarm();
    } else { // in clock mode

      hh24++;

      if (hh24 > 23) {
        hh24 = 0;
      }
      updateRTC();
    }
  }

  //minutes button pressed
  if (digitalRead(minsButton) == LOW) {

    //debounce
    delay(200);

    if (true == inAlarmMenu) {
      //set alarm minutes
      alarmMinute++;

      if (alarmMinute > 59) {
        alarmMinute = 0;
      }
      setAlarm();
    } else { // in clock mode
      mm++;
      ss = 0;

      if (mm > 59) {
        mm = 0;
      }
      updateRTC();
    }
  }

  //alarm menu button pressed
  if (digitalRead(alarmMenu) == LOW) {

    //debounce
    delay(200);

    inAlarmMenu = !inAlarmMenu; //toggle alarm menu / clock mode
  }

  //alarm enable button pressed
  if (digitalRead(alarmEnable) == LOW) {

    //debounce
    delay(200);

    //enable / disable the alarm
    if (false == setAlarmActive) {
      setAlarmActive = true; //turn on alarm indicator
      clock.armAlarm1(true); //arm alarm
    } else {
      setAlarmActive = false; //turn on alarm indicator
      if (clock.isArmed1()) {
        clock.armAlarm1(false); //disarm alarm.
      }
    }
  }

  //kill alarm
  if (clock.isAlarm1()) {
    clock.isAlarm1(false);
  }

  //end loop
}

//get character from segment array
void displayDigit(byte seg) {

  digitalWrite(0, segArray[seg][0]); //A
  digitalWrite(1, segArray[seg][1]); //B
  digitalWrite(2, segArray[seg][2]); //C
  digitalWrite(3, segArray[seg][3]); //D
  digitalWrite(4, segArray[seg][4]); //E
  digitalWrite(5, segArray[seg][5]); //F
  digitalWrite(6, segArray[seg][6]); //G

}

//RTC stuff
void updateRTC() {
  // save date/time (YYYY, MO, DD, HH, MM, SS
  clock.setDateTime(2016, 1, 1, hh24, mm, ss); //just set date to 01/01/2016 since I'm not using it on the display
}

void setAlarm() {
  clock.clearAlarm1();
  // setAlarm1(Date or Day, Hour, Minute, Second, Mode, Armed = true)
  clock.setAlarm1(0, alarmHour, alarmMinute, 0, DS3231_MATCH_H_M_S);
}

